<?php

namespace TDevAgency\Cms\Models\BaseModels;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Kyslik\ColumnSortable\Sortable;

final class AdminUser extends Authenticatable
{
    use Notifiable, Sortable;

    protected $sortable = [
        'id',
        'name',
        'is_active',
        'email',
        'created_at',
        'updated_at'
    ];
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email', 'password', 'lang', 'is_active'
    ];
    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];
    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];
    protected $with = [
        'roles'
    ];

    /**
     * @return HasMany
     */
    public function roles(): HasMany
    {
        return $this->hasMany(AdminUserRole::class)
            ->where('role_status', 1);
    }

    /**
     * @param Builder $query
     * @param int $perPage
     * @return LengthAwarePaginator
     */
    public function scopeGetList(Builder $query, $perPage = 15): LengthAwarePaginator
    {
        return $query->where('id', '!=', Auth::id())
            ->whereHas('roles', function (Builder $query) {
                $roles = [];

                foreach (Auth::user()->roles as $role) {

                    if ($role instanceof AdminUserRole) {
                        $roles = array_merge($roles, Config::get('cms.roles.' . $role->admin_role . '.users_access'));
                    } else {
                        $roles = array_merge($roles, Config::get('cms.roles' . $role . '.users_access'));
                    }
                }

                return $query->whereIn('admin_role', array_unique($roles));
            })
            ->sortable()->paginate($perPage);
    }

}
